// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.

//
// File:      OmImageDisplay.cc
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)r149
// Generated: Fri Sep  6 12:10:40 MEST 2002
// Usage:     -
// Info:      -

#include "OmImageDisplay.h"
#include <fstream>
#include <sstream>
//#include <unistd.h> // unlink()
#include <cstdio>
#include "AngleFormatter.h"
#include <qstring.h>
#include "OmSourceRegions.h"
#include "OmSourceUtility.h"
#include "param.h"
#include <cmath>
extern F90MODVARDEF(OmSourceParameters, omsource_glue, mybackgroundsources);
extern F90MODVARDEF(OmSourceParameters, omsource_glue, mysources);
extern F90MODVARDEF(OmSourceRegions, omsource_glue, mybackgroundregions);

F90MODPROCPROTO(void, omsource_glue, createtheregionfile)();

char OmImageDisplay::_regionFile[100] = "\0";


/***************************************************************
*
****************************************************************/
void OmImageDisplay::toRegion(const OmSourceParameters &sources, 
                           const char* colour, const bool deleteRegions)
{
 


    if (deleteRegions) send("regions deleteall");
    if (sources.ubound() < 1) return;
  
    string col = colour;
    if ( col.find("DEF") != string::npos)
    {
   
        setParameter( "wdxset", string( _regionFile) );

        F90MODPROC( omsource_glue, createtheregionfile )();
        send("regions file " + string( _regionFile ));
 

        return;
    }
    

    ofstream os( _regionFile );

    //    os << "global color=green select=0 edit=0 move=0 delete=1" << endl;

     os << "global select=0 edit=0 move=0 delete=1" << endl;




   
    for (int id = sources.lbound(); id <= sources.ubound(); id++) 
    {

     
        OmSourceParameter& s = sources[id];   

        if (abs(s.semia) > 100.0 || isNanValue(s.semia)) 
	{
	    s.semia = 2.0;
            s.semib = 2.0;
	    //         s.alpha = 0.0;
            s.theta = 0.0;
 
        } 




	//	 	 cout << "semi a=" << s.semia << " semi b=" << s.semib << " theta=" << s.alpha << endl;
 
        
	os << " ellipse("
	   << s.xcoord << ','
	   << s.ycoord << ','
	   << s.semia * 3.0  << ','
	   << s.semib * 3.0 << ','
	  //	   << s.alpha << ')'  // Position angel of ellipse 
           << s.theta << ')'  // Position angel of ellipse 
 
	   << " #text='" << id << "' color=" << colour 
	   << endl;

      


    }
   

    os.close();



    send("regions file " + string( _regionFile ));
   
   
}
/***************************************************************
*
****************************************************************/
void OmImageDisplay::toRegion1(const OmSourceParameters &sources, 
                           const char *colourIn, const bool deleteRegions)
{


    if (deleteRegions) send("regions deleteall");
    if (sources.ubound() < 1) return;
  
 
    ofstream os( _regionFile );

    os << "global select=0 edit=0 move=0 delete=1" << endl;


    string colour1 = colourIn;
    bool fixedColour = true;
    if ( colour1.find("DEF") != string::npos)
    {
   
      fixedColour = false;
    }
    
     
    for (int id = sources.lbound(); id <= sources.ubound(); id++) 
    {             
        OmSourceParameter& s = sources[id];   
       
        string colour;

        if(fixedColour)
	{
	    colour = colourIn;
        }
        else
	{
	    if( (int)s.extensionflag == 0)
	    {
	        colour = "green";
   
	    

	    }
            else
	    {
	        colour = "cyan";
            } 
        }

        os << "point(" << s.xcoord << ", " << s.ycoord << ") # point=cross color=" << colour << " text={" << id << "}"
	   << endl;

      

   
    }
   

    os.close();



    send("regions file " + string( _regionFile ));
   
   
}





/***************************************************************
*
****************************************************************/
void OmImageDisplay::displayBackgroundRegions(const OmSourceParameters& sources)
{
           

    ofstream os(_regionFile);

    //    os << "global color=green select=0 edit=0 move=0 delete=1" << endl;
   os << "global select=0 edit=0 move=0 delete=1" << endl;


   //          cout << "OmImageDisplay Line 109 display background regions\n";   

    for (int id = sources.lbound(); id <= sources.ubound(); id++) 
    {


        OmSourceParameter& s = sources[id];   


        if (abs(s.semia) > 100.0 || isNanValue(s.semia)) 
	{
	  s.semia = 2.0;
          s.semib = 2.0;
	  //          s.alpha = 0.0; 
          s.theta = 0.0; 

        } 
   
        
	os << " ellipse("
	   << s.xcoord << ','
	   << s.ycoord << ','
	   << s.semia * 3.0  << ','
	   << s.semib * 3.0 << ','
	  //	   << s.alpha << ')'  // Position angel of ellipse
           << s.theta << ')'  // Position angel of ellipse
  
	   << " #text='" << id << "' color=green" 
	   << endl;

	//	 cout << id << ") x=" << s.xcoord << " y=" << s.ycoord << endl;
    }
   

    os.close();
 
    send("regions file " + string(_regionFile));
   
  


}
/***************************************************************
*
****************************************************************/
void OmImageDisplay::displayBackgroundRegions(const OmSourceRegions*
     sources, const float radius)
{
       

    ofstream os(_regionFile);

 


    //    os << "global color=green select=0 edit=0 move=0 delete=1" << endl;

   os << "global select=0 edit=0 move=0 delete=1" << endl;



    //        cout << "Line 161 display backgroudn regions\n";   

   
    for (int i= sources->lbound(); i<= sources->ubound(); i++) 
    {

           os << " circle("
	       
	   << (*sources)[i].x() << ','
	   << (*sources)[i].y() << ','
	   << radius << ')'  // Position angel of ellipse  
	   << " #text='" << i << "' color=green" 
	   << endl;

	   //	cout << i << ") x=" << (*sources)[i].x() << " y=" << (*sources)[i].y() << endl;

    }
   

    os.close();
   

    send("regions file " + string(_regionFile));
   
    remove(_regionFile);


}


/***************************************************************
*
****************************************************************/
void OmImageDisplay::displaySourceRegions(const OmSourceParameters& sources)
{
  
  //           cout << "Line 235 displaysourceregions\n";   




    if ( sources.ubound()  < 1) return;


    ofstream os(_regionFile);

    //    os << "global color=blue select=0 edit=0 move=0 delete=1" << endl;
   
  os << "global select=0 edit=0 move=0 delete=1" << endl;
  



    for (int id = sources.lbound(); id <= sources.ubound(); id++) 
    {


        OmSourceParameter& s = sources[id];   
        if (abs(s.semia) > 100.0 || isNanValue(s.semia)) 
	{
	  s.semia = 2.0;
          s.semib = 2.0;
	  //          s.alpha = 0.0;
          s.theta = 0.0;
 
        } 
  

        
	os << " ellipse("
	   << s.xcoord << ','
	   << s.ycoord << ','
	   << s.semia * 3.0  << ','
	   << s.semib * 3.0 << ','
	  //	   << s.alpha << ')'  // Position angel of ellipse
           << s.theta << ')'  // Position angel of ellipse
  
	   << " #text='" << id << "' color=green" 
	   << endl;



	//	 cout << "semi a=" << s.semia << " semi b=" << s.semib << " theta=" << s.alpha << endl;
 


    }
   

    os.close();
    //    send("regions color green");
    send("regions file " + string(_regionFile));
   


}
/***************************************************************
*
****************************************************************/
void OmImageDisplay::displaySourceRegions(const OmSourceRegions*
     sources, const float radius)
{
   
 
  //   cout << "Line 244\n";   

    ofstream os(_regionFile);

    //    os << "global color=blue select=0 edit=0 move=0 delete=1" << endl;
   
  os << "global select=0 edit=0 move=0 delete=1" << endl;
  

    for (int i= sources->lbound(); i<= sources->ubound(); i++) 
    {


           os << " circle("
	       
	   << (*sources)[i].x() << ','
	   << (*sources)[i].y() << ','
	   << radius << ')'  // Position angel of ellipse  
	   << " #text='" << i << "' color=green" 
	   << endl;

    }
   

    os.close();
    //    send("regions color green");
    send("regions file " + string(_regionFile));
   
    remove(_regionFile);


}
/***************************************************************
*
****************************************************************/
void OmImageDisplay::displaySourceRegions(const OmSourceRegions*
     sources, const vector<float> &radius, const char *colour)
{
   
  //     cout << "Line 325 displaysourceregions \n";   

    ofstream os(_regionFile);

    //    os << "global color=blue select=0 edit=0 move=0 delete=1" << endl;
   
   os << "global select=0 edit=0 move=0 delete=1" << endl;
   





    for (int i= sources->lbound(); i <= sources->ubound(); i++) 
    {

      
           
           os << " circle("
	       
	   << (*sources)[i].x() << ','
	   << (*sources)[i].y() << ','
 

	   << radius[i - 1] << ')'  // radius of circle  

	      << " #text='" << i << "' color=" << colour 
	   << endl;
	  


    }
   
    //   cout << "Line 364 displaysourceregions \n";   


    os.close();
    //    send("regions color green");
    send("regions file " + string(_regionFile));
   
    //  cout << "Line 371 displaysourceregions \n";   



}

 




/***************************************************************
*
****************************************************************/
void OmImageDisplay::displayBackgroundRegions( const vector<OmSourceParameter> &sourceList,
                           const char *colour)
{
   

    ofstream os(_regionFile);


   
    os << "global select=0 edit=0 move=0 delete=1" << endl;
   
    



    for (unsigned i= 0; i < sourceList.size(); i++) 
    {

        os << " circle("
	       
	   << sourceList[i].xcoord << ','
	   << sourceList[i].ycoord << ','
 

	   << sourceList[i].photometryradius << ')'  // radius of circle  

	   << " #text='" << (i + 1) << "' color=" << colour 
	   << endl;
	  


    }
   

    os.close();
  
    send("regions file " + string(_regionFile));
   

}


/***************************************************************
*
****************************************************************/
void OmImageDisplay::deleteRegions(const OmSourceRegions* sources)
{
   
  
    for (int i= sources->lbound(); i<= sources->ubound(); i++) 
    {


      //        OmSourceRegions & s = (*sources)[id];   

      char text[100];
      sprintf(text, "delete %f7.2 %f7.2", (*sources)[i].x(), (*sources)[i].y());
      send(text);	   

    }
   
}

/******************************************************
*
******************************************************/
void OmImageDisplay::zoomTo(const OmSourceParameter& s, int zoomFactor, const char* colour1, const char *colour2)
{
    std::stringstream ss;
   
    send("regions deleteall");
    ss << "pan to " << s.xcoord << ' ' << s.ycoord << " image";
    send(ss.str());
   



    if(zoomFactor > 0)
    {
    
        char text[12]; 
        sprintf(text, "zoom to %d", zoomFactor);
        send(text);  // was 12
    }   
  


    ofstream os(_regionFile);
    float semiA = s.photometryradius * s.axialRatio;
 
   os << "global select=0 edit=0 move=0 delete=1" << endl;
 

    os << " ellipse("
       << s.xcoord << ','
       << s.ycoord << ','
       << semiA  << ','
       << s.photometryradius << ','
       << s.alpha   // Position angel of ellipse  
       << ')' << " #color=" << colour1 
       << endl;

    semiA = s.innerbackgroundradius * s.axialRatio;
    os << " ellipse("
       << s.xcoord << ','
       << s.ycoord << ','
       << semiA    << ','  
       << s.innerbackgroundradius  << ',' 
       << s.alpha   // Position angel of ellipse  
       << ')' << " #color=" << colour2 
       << endl;
  
    semiA = s.outerbackgroundradius * s.axialRatio;
    os << " ellipse("
       << s.xcoord << ','
       << s.ycoord << ','
       << semiA    << ','  
       << s.outerbackgroundradius  << ',' 
       << s.alpha   // Position angel of ellipse  
       << ')' << " #color=" << colour2 
       << endl;
  
  

    os.close();
    // Send to ds9 
    send("regions file " + string(_regionFile));
     
}


/******************************************************
*
******************************************************/
void OmImageDisplay::zoom1To(const OmSourceParameter& s, int zoomFactor, const char* colour1)
{
    std::stringstream ss;
   
    send("regions deleteall");
    ss << "pan to " << s.xcoord << ' ' << s.ycoord << " image";
    send(ss.str());
   



    if(zoomFactor > 0)
    {
    
        char text[12]; 
        sprintf(text, "zoom to %d", zoomFactor);
        send(text);  // was 12
    }   
  


    ofstream os(_regionFile);
    float semiA = s.photometryradius * s.axialRatio;
 
   os << "global select=0 edit=0 move=0 delete=1" << endl;
 

    os << " ellipse("
       << s.xcoord << ','
       << s.ycoord << ','
       << semiA  << ','
       << s.photometryradius << ','
       << s.alpha   // Position angel of ellipse  
       << ')' << " #color=" << colour1 
       << endl;


    os.close();
    // Send to ds9 
    send("regions file " + string(_regionFile));
     
}




/******************************************************
*
******************************************************/
void OmImageDisplay::displaySource(const int number, const OmSourceParameter& s1)
{
    std::stringstream ss;
    OmSourceParameter s = s1;   

    

    send("regions deleteall");
    ss << "pan to " << s.xcoord << ' ' << s.ycoord << " image";
    send(ss.str());
    send("zoom to 15");

    //     cout << "Line 409\n";   


    ofstream os(_regionFile);

 

    if (abs(s.semia) > 100.0 || isNanValue(s.semia)) 
	{
	  s.semia = 2.0;
          s.semib = 2.0;
	  //          s.alpha = 0.0; 
          s.theta = 0.0;
        } 
  




    os << " ellipse("
       << s.xcoord << ','
       << s.ycoord << ','
       << s.semia * 3.0 << ','
       << s.semib * 3.0 << ','
       << s.theta  

       << ')'
       << " # color=cyan"
       << endl;


  
  
    AngleFormatter af;
    af.setMode(AngleFormatter::HMS);
    QString text1(af.asString(s.rightascension).c_str());

    af.setMode(AngleFormatter::DMS);
    QString text2(af.asString(s.declination).c_str());
  

    QString text;
    text.sprintf("'%d) RA=%s, DEC=%s'", number, (const char*)text1, (const char*)text2);
  
    os  << " text(" <<  (s.xcoord - 2.0)  << " "  << (s.ycoord + 12.0) << " " 
        <<  ((const char *)text) << " # color=red font='times 20 bold'" << endl;
      
    text.sprintf("'Magnitude= %5.2f+/-%5.2f'", s.magnitude, s.magnitudeerror);
    os  << " text(" <<  (s.xcoord - 4.0)  << " "  << (s.ycoord + 10.0) << " " 
        <<  ((const char *)text) << " # color=red font='times 20 bold'" << endl;
  

    os.close();
 
    send("regions file " + string(_regionFile));
          
}
/**********************************************************
*
***********************************************************/
void OmImageDisplay::plotPoint(const OmSourceParameter& s)
{


    std::stringstream ss;

    ss << "point(" << s.xcoord << " " << s.ycoord << ") # point=cross" << endl;
  
    send(ss.str());
  
 
   std::stringstream ss1;

    ss1 << " x point " << s.xcoord << ',' << s.ycoord;
    //  ss1 << " x point " << s.xcoord << ',' << s.ycoord;

    //    send(ss1.str());




}
/******************************************************************************
*
* Displays the current source-list regions
*****************************************************************************/
void OmImageDisplay::displayCurrentSourceList( const char* colour, const bool deleteRegions )
{

 
    OmSourceParameters& sources = F90MODVAR(omsource_glue, mysources);
     

    toRegion( sources, colour, deleteRegions );

 
}
/******************************************************************************
*
* Displays the current source-list regions
*****************************************************************************/
void OmImageDisplay::displayCurrentSourceList1( const char* colour, const bool deleteRegions )
{

 
    OmSourceParameters& sources = F90MODVAR(omsource_glue, mysources);
     

    toRegion1( sources, colour, deleteRegions );

 
}
