// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// -*-C++-*-
//
// File:      OmSourceRegions.h
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)
// Generated: Tue Sep 10 10:58:50 CEST 2002
// Usage:     interface of C++/f90
// Info:      

#ifndef OmSourceRegions_h
#define OmSourceRegions_h
#include "Region.h"     // Point
#include "DopeVector.h"
#include <cstring>
// forward declaration of f90-methods
class OmSourceRegions;
class OmImageDisplay;
F90MODPROCPROTO(void,omsourceregion_mod,new_)(OmSourceRegions*,const int&, const int&);
F90MODPROCPROTO(void,omsourceregion_mod,delete_)(OmSourceRegions*);


// A Point<float> is equivalent to f90::type(OmSourceRegion)
class OmSourceRegions : public DopeVector<SelectlibRegion::Point<float> > 
{
public:
  OmSourceRegions(int length)
  {
    int lbound = 1;
    int ubound = length;
    F90MODPROC(omsourceregion_mod, new_)(this, lbound, ubound);
  }
  OmSourceRegions(int lbound,int ubound)
  {
      F90MODPROC(omsourceregion_mod, new_)(this, lbound, ubound);
  }

  OmSourceRegions(OmSourceRegions *regions)
  {


        F90MODPROC(omsourceregion_mod, new_)(this, regions->lbound(), regions->ubound());
        

        for (int i = regions->lbound(); i <= regions->ubound(); i++)
	   {
             (*this)[i] = (*regions)[i];           
          	              
	   }         
   
   }




  ~OmSourceRegions()
  {
      F90MODPROC(omsourceregion_mod, delete_)(this);
  }
 
   
  
private:
   // disabled member functions
  OmSourceRegions();
  OmSourceRegions(const OmSourceRegions&);
  OmSourceRegions& operator=(const OmSourceRegions&);
   

  

};
class OmImageRegions
{
public:
    OmImageRegions() {};
    ~OmImageRegions() {};

    static OmSourceRegions* GetRegions(   OmImageDisplay*  _display); 
    static OmSourceRegions* GetNewRegions(   OmImageDisplay*  _display, float nx, float ny); 
    static OmSourceRegions* GetOldRegions() {return _oldRegions;}; 
    static void setColour( string colour) { _colour = colour; };
    static OmSourceRegions* _oldRegions;  
    static OmSourceRegions* _backgroundRegions;  
    
    static int _noRegions;
    static int _noBackgroundRegions;
    static string _colour;
    static void updateBackgroundRegions(OmSourceRegions*);
    static void deleteBackgroundRegions(OmImageDisplay*  _display);

  static void deleteRegion(int no, OmImageDisplay*  _display); 
  static void deleteAllRegions(); 
};



#endif
