// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.

#include <qinputdialog.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qstring.h>

#include "qapplication.h"
#include "qmessagebox.h"
#include <param.h>
#include "Dal.h"
#include "allocate.h"
#include "OmSourceUtility.h"
#include <cstdlib>






bool isNanValue(float x)
{

  stringstream s;
  s << x;

  if (s.str() == "nan" || s.str() == "NAN")
    return true;
  else
    return false;



}




/************************************************************
* GetOldFileName
*************************************************************/
void GetOldFileName(QWidget *parent, char oldFileName[], 
           const char *parameter, const char *title, const QString filterList[])
{ 
    string fileName = stringParameter(parameter);
    QFileDialog fd(QString::null, filterList[0], 
                       parent , title);
    fd.setCaption(title);
    fd.setMode(QFileDialog::ExistingFile);
   int i = 1;
   while(filterList[i])
   {

       fd.addFilter(filterList[i++]);
   }
 

       
    if (fileName == string("") || !QFile::exists((const QString)fileName.c_str()))
    {

        /**************************************************
	* Display get file name dialog
	*************************************************/ 

        bool okay = FALSE;
        QString text;
   
        while (!okay)
	{
	    QApplication::beep();        
 
            if ( fd.exec() == QDialog::Accepted )
	      text = fd.selectedFile();

         
              
              if ( text.isEmpty() || !QFile::exists((const QString)text))
            {  
	
                int ret = QMessageBox::critical(0, "No existing file has been selected",
					      "Do you wish to try again (yes), or exit (No/Cancel)",
					      QMessageBox::Yes | QMessageBox::Default,
                                              QMessageBox::No,
                                              QMessageBox::Cancel | QMessageBox::Escape );
              
              switch (ret)
	      {
	      case   (QMessageBox::Yes):
		      okay = FALSE; 
		      break;
	      case   (QMessageBox::No):
		      exit(0); 
		      break;
	      case   (QMessageBox::Cancel):
		      exit(0); 
		      break;
	      default:
		break;
	      }
            }
            else
	 break;
                     
        }
                      
        fileName = string((const char *)text);
     
    }
        
    strcpy(oldFileName, fileName.c_str());
    
}
/************************************************************
* GetNewFileName
*************************************************************/
 
bool GetNewFileName(QWidget *parent, char newFileName[], 
            const char *parameter, const QString title, const QString caption)
{ 
    string fileName = stringParameter(parameter);
    

    bool okay = FALSE;

    if (fileName  != string(""))
    {

        if (!QFile::exists(fileName.c_str())) return true;
        
        int ret = QMessageBox::critical(0, "File already exists",
		     "Do you wish to type in another file (yes/no), or exit (Cancel)",
		     QMessageBox::Yes | QMessageBox::Default,
                     QMessageBox::No,
                     QMessageBox::Cancel | QMessageBox::Escape );
            switch (ret)
	    {
	      case   (QMessageBox::Yes):
		      okay = FALSE; 
                      
		      break;
	      case   (QMessageBox::No):
		     
		      strcpy(newFileName, fileName.c_str());
                      return true;
		      break;
	      case   (QMessageBox::Cancel):
   

		      return false; 
		      break;
	      default:
		break;
	    }
    }

    
        
    while (!okay)
    {

        QString text = QInputDialog::getText(
                            title, caption, QLineEdit::Normal,
                            QString::null, &okay, parent );
    

        if (okay && text.isEmpty()) {
	  okay = false;
            continue;
        } 
	if (!okay) return false;    

        fileName = string((const char*)text);
                       
        if (QFile::exists(fileName.c_str()))
	{
	    int ret = QMessageBox::critical(0, "File already exists",
		     "Do you wish to type in another file (yes/no), or exit (Cancel)",
		     QMessageBox::Yes | QMessageBox::Default,
                     QMessageBox::No,
                     QMessageBox::Cancel | QMessageBox::Escape );
            switch (ret)
	    {
	      case   (QMessageBox::Yes):
		      okay = FALSE; 
		      break;
	      case   (QMessageBox::No):
		      okay = TRUE;
		       
		      break;
	      case   (QMessageBox::Cancel):


		      return false; 
		      break;
	      default:
		break;
	    }
	 }

	
              
    }
     
 
    strcpy(newFileName, fileName.c_str());
    return true;
}
/****************************************************
* LoadImage
****************************************************/
float** loadImage(char *fileName, int &nx, int &ny)
{
    DataSet* set = dataSetServer->open(fileName, Dal::Read);
    Array* array = set->array("PRIMARY");
    longVec arraySize = array->size();

    nx = arraySize[1];
    ny = arraySize[0];

   cout << "laodimage line 195  nx=" << nx << " ny=" << ny << endl;

    
    ArrayData *arrayData = array->data();
    real32 * data = arrayData->real32Data();

    float **image = NULL;
    allocate( &image, nx, ny ); 
    int k = 0; 




    cout << "Line 208\n";
    for (int j = 0; j < ny; j++)
        for (int i = 0; i < nx; i++)
            image[i][j] = data[ k++ ];
                
    delete arrayData;

  cout << "Line 215\n";


    dataSetServer->close(set);
    cout << "laodimage 210 nx=" << nx << " ny=" << ny << endl;
    return image;
}
