// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/**********************************
* ComputeAverage
* Calculates the weighted average and
* standard deviation 
**********************************/
#include "average.h"
#include <cmath>

float Average::ComputeAverage(const int size, 
              const float* x, const float* dx, 
              float &standardDeviation) 

{    
       
    float mean;  
    
      
    if (size == 1) 
    {
        mean = x[0];
	standardDeviation = dx[0];
	return mean;
    }


      
    
    for (int i = 0; i < size; i++)
    { 
       
        double weight = 1.0 / (dx[i] * dx[i]);
	sum1 += weight;
	sum2 += x[i] * weight;
	sum3 += weight * weight;
    

    }


    mean = sum2 / sum1;  // The mean

  

    for (int i = 0; i < size; i++)
    {
       
   
        double difference = x[i] - mean;       
    
	double weight = 1.0 / (dx[i] * dx[i]);
        sum4 += weight * difference * difference;
       
       
    }

    double d = sum1 - sum3 / sum1;
   
    
    standardDeviation = sqrt(sum4 / d);  //  The standard deviation
    
    return mean;      
}
/*******************************************************
*
*******************************************************/
float Average::UpdateAverage(const float x, 
             const float dx, float& standardDeviation)
{
 
    double weight = 1.0 / dx * dx;

    sum1 += weight;
    sum2 += x * weight;
    sum3 += weight * weight;
    

    float mean = sum2 / sum1;  // The mean

   
    double difference = x - mean;       
    
    sum4 += weight * difference * difference;
       
      
    double d = sum1 - sum3 / sum1;
   
    if (d > 0.0)    
        standardDeviation = sqrt(sum4 / d);  //  The standard deviation
    else
        standardDeviation = 0.0;

    return mean;      
}

float Average::UpdateAverage(const int size, 
              const float* x, const float* dx, 
              float &standardDeviation) 

{    
       
    float mean;  
    
           
    
    for (int i = 0; i < size; i++)
    { 
       
        double weight = 1.0 / (dx[i] * dx[i]);
	sum1 += weight;
	sum2 += x[i] * weight;
	sum3 += weight * weight;
    

    }


    mean = sum2 / sum1;  // The mean

  

    for (int i = 0; i < size; i++)
    {
       
   
        double difference = x[i] - mean;       
    
	double weight = 1.0 / (dx[i] * dx[i]);
        sum4 += weight * difference * difference;
       
       
    }

    double d = sum1 - sum3 / sum1;
   
   
  
    if (d > 0.0)    
        standardDeviation = sqrt(sum4 / d);  //  The standard deviation
    else
        standardDeviation = 0.0;



    
    return mean;      
}

